﻿using System;
using System.IO;
using System.Net;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// IP 헤더
    /// </summary>
    public class IPHeader
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 버전/헤더 길이
        /// </summary>
        private byte versionAndHeaderLength;

        /// <summary>
        /// 차별화 서비스
        /// </summary>
        private byte differentiatedServices;

        /// <summary>
        /// 전체 길이
        /// </summary>
        private ushort totalLength;

        /// <summary>
        /// 식별
        /// </summary>
        private ushort identification;

        /// <summary>
        /// 플래그/오프셋
        /// </summary>
        private ushort flagsAndOffset;

        /// <summary>
        /// TTL
        /// </summary>
        private byte ttl;

        /// <summary>
        /// 프로토콜
        /// </summary>
        private byte protocol;

        /// <summary>
        /// 체크썸
        /// </summary>
        private short checksum;

        /// <summary>
        /// 소스 주소
        /// </summary>
        private uint sourceAddress;

        /// <summary>
        /// 목적 주소
        /// </summary>
        private uint destinationAddress;

        /// <summary>
        /// 헤더 길이
        /// </summary>
        private byte headerLength; 

        /// <summary>
        /// 데이터 바이트 배열
        /// </summary>
        private byte[] dataByteArray = new byte[4096];

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 버전 - Version

        /// <summary>
        /// 버전
        /// </summary>
        public string Version
        {
            get
            {
                if((this.versionAndHeaderLength >> 4) == 4)
                {
                    return "IP v4";
                }
                else if((this.versionAndHeaderLength >> 4) == 6)
                {
                    return "IP v6";
                }
                else
                {
                    return "Unknown";
                }
            }
        }

        #endregion
        #region 헤더 길이 - HeaderLength

        /// <summary>
        /// 헤더 길이
        /// </summary>
        public string HeaderLength
        {
            get
            {
                return this.headerLength.ToString();
            }
        }

        #endregion
        #region 메시지 길이 - MessageLength

        /// <summary>
        /// 메시지 길이
        /// </summary>
        public ushort MessageLength
        {
            get
            {
                return (ushort)(this.totalLength - this.headerLength);
            }
        }

        #endregion
        #region 차별화 서비스 - DifferentiatedServices

        /// <summary>
        /// 차별화 서비스
        /// </summary>
        public string DifferentiatedServices
        {
            get
            {
                return string.Format("0x{0:x2} ({1})", this.differentiatedServices, this.differentiatedServices);
            }
        }

        #endregion
        #region 플래그 - Flags

        /// <summary>
        /// 플래그
        /// </summary>
        public string Flags
        {
            get
            {
                int flags = this.flagsAndOffset >> 13;

                if(flags == 2)
                {
                    return "Don't fragment";
                }
                else if(flags == 1)
                {
                    return "More fragments to come";
                }
                else
                {
                    return flags.ToString();
                }
            }
        }

        #endregion
        #region 단편화 오프셋 - FragmentationOffset

        /// <summary>
        /// 단편화 오프셋
        /// </summary>
        public string FragmentationOffset
        {
            get
            {
                int offset = this.flagsAndOffset << 3;

                offset >>= 3;

                return offset.ToString();
            }
        }

        #endregion
        #region TTL - TTL

        /// <summary>
        /// TTL
        /// </summary>
        public string TTL
        {
            get
            {
                return this.ttl.ToString();
            }
        }

        #endregion
        #region 프로토콜 타입 - ProtocolType

        /// <summary>
        /// 프로토콜 타입
        /// </summary>
        public Protocol ProtocolType
        {
            get
            {
                if(this.protocol == 6)
                {
                    return Protocol.TCP;
                }
                else if(this.protocol == 17)
                {
                    return Protocol.UDP;
                }
                else
                {
                    return Protocol.UNKNOWN;
                }
            }
        }

        #endregion
        #region 체크썸 - Checksum

        /// <summary>
        /// 체크썸
        /// </summary>
        public string Checksum
        {
            get
            {
                return string.Format("0x{0:x2}", this.checksum);
            }
        }

        #endregion
        #region 소스 주소 - SourceAddress

        /// <summary>
        /// 소스 주소
        /// </summary>
        public IPAddress SourceAddress
        {
            get
            {
                return new IPAddress(this.sourceAddress);
            }
        }

        #endregion
        #region 목적 주소 - DestinationAddress

        /// <summary>
        /// 목적 주소
        /// </summary>
        public IPAddress DestinationAddress
        {
            get
            {
                return new IPAddress(this.destinationAddress);
            }
        }

        #endregion
        #region 전체 길이 - TotalLength

        /// <summary>
        /// 전체 길이
        /// </summary>
        public string TotalLength
        {
            get
            {
                return this.totalLength.ToString();
            }
        }

        #endregion
        #region 식별 - Identification

        /// <summary>
        /// 식별
        /// </summary>
        public string Identification
        {
            get
            {
                return this.identification.ToString();
            }
        }

        #endregion
        #region 데이터 바이트 배열 - DataByteArray

        /// <summary>
        /// 데이터 바이트 배열
        /// </summary>
        public byte[] DataByteArray
        {
            get
            {
                return this.dataByteArray;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - IPHeader(sourceByteArray, countReceived)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sourceByteArray">소스 바이트 배열</param>
        /// <param name="countReceived">수신 카운트</param>
        public IPHeader(byte[] sourceByteArray, int countReceived)
        {
            try
            {
                MemoryStream memoryStream = new MemoryStream(sourceByteArray, 0, countReceived);
                BinaryReader binaryReader = new BinaryReader(memoryStream);

                this.versionAndHeaderLength = binaryReader.ReadByte();
                this.differentiatedServices = binaryReader.ReadByte();
                this.totalLength            = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.identification         = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.flagsAndOffset         = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.ttl                    = binaryReader.ReadByte();
                this.protocol               = binaryReader.ReadByte();
                this.checksum               = IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.sourceAddress          = (uint)(binaryReader.ReadInt32());
                this.destinationAddress     = (uint)(binaryReader.ReadInt32());
                this.headerLength           = this.versionAndHeaderLength;
                this.headerLength <<= 4;
                this.headerLength >>= 4;
                this.headerLength *=  4;

                Array.Copy(sourceByteArray, this.headerLength, this.dataByteArray, 0, this.totalLength - this.headerLength); 
            }
            catch(Exception exception)
            {
                MessageBox.Show
                (
                    exception.Message,
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            }
        }

        #endregion
    }
}