﻿using System;
using System.Net;
using System.IO;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// TCP 헤더
    /// </summary>
    public class TCPHeader
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 포트
        /// </summary>
        private ushort sourcePort;

        /// <summary>
        /// 목적 포트
        /// </summary>
        private ushort destinationPort;

        /// <summary>
        /// 시퀀스 번호
        /// </summary>
        private uint sequenceNumber = 555;

        /// <summary>
        /// 승인 번호
        /// </summary>
        private uint acknowledgementNumber = 555;

        /// <summary>
        /// 데이터 오프셋/플래그
        /// </summary>
        private ushort dataOffsetAndFlags = 555;

        /// <summary>
        /// 윈도우 크기
        /// </summary>
        private ushort windowSize = 555;

        /// <summary>
        /// 체크썸
        /// </summary>
        private short checksum = 555;

        /// <summary>
        /// 긴급 포인터
        /// </summary>
        private ushort urgentPointer;      

        /// <summary>
        /// 헤더 길이
        /// </summary>
        private byte headerLength;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 소스 포트 - SourcePort

        /// <summary>
        /// 소스 포트
        /// </summary>
        public string SourcePort
        {
            get
            {
                return this.sourcePort.ToString();
            }
        }

        #endregion
        #region 목적 포트 - DestinationPort

        /// <summary>
        /// 목적 포트
        /// </summary>
        public string DestinationPort
        {
            get
            {
                return this.destinationPort.ToString();
            }
        }

        #endregion
        #region 시퀀스 번호 - SequenceNumber

        /// <summary>
        /// 시퀀스 번호
        /// </summary>
        public string SequenceNumber
        {
            get
            {
                return this.sequenceNumber.ToString();
            }
        }

        #endregion
        #region 승인 번호 - AcknowledgementNumber

        /// <summary>
        /// 승인 번호
        /// </summary>
        public string AcknowledgementNumber
        {
            get
            {
                if((this.dataOffsetAndFlags & 0x10) != 0)
                {
                    return this.acknowledgementNumber.ToString();
                }
                else
                {
                    return string.Empty;
                }
            }
        }

        #endregion
        #region 헤더 길이 - HeaderLength

        /// <summary>
        /// 헤더 길이
        /// </summary>
        public string HeaderLength
        {
            get
            {
                return this.headerLength.ToString();
            }
        }

        #endregion
        #region 윈도우 크기 - WindowSize

        /// <summary>
        /// 윈도우 크기
        /// </summary>
        public string WindowSize
        {
            get
            {
                return this.windowSize.ToString();
            }
        }

        #endregion
        #region 플래그 - Flags

        /// <summary>
        /// 플래그
        /// </summary>
        public string Flags
        {
            get
            {
                int flags = this.dataOffsetAndFlags & 0x3F;

                string flagsString = string.Format("0x{0:x2} (", flags);

                if((flags & 0x01) != 0)
                {
                    flagsString += "FIN, ";
                }

                if((flags & 0x02) != 0)
                {
                    flagsString += "SYN, ";
                }

                if((flags & 0x04) != 0)
                {
                    flagsString += "RST, ";
                }

                if((flags & 0x08) != 0)
                {
                    flagsString += "PSH, ";
                }

                if((flags & 0x10) != 0)
                {
                    flagsString += "ACK, ";
                }

                if((flags & 0x20) != 0)
                {
                    flagsString += "URG";
                }

                flagsString += ")";

                if(flagsString.Contains("()"))
                {
                    flagsString = flagsString.Remove(flagsString.Length - 3);
                }
                else if(flagsString.Contains(", )"))
                {
                    flagsString = flagsString.Remove(flagsString.Length - 3, 2);
                }

                return flagsString;
            }
        }

        #endregion
        #region 체크썸 - Checksum

        /// <summary>
        /// 체크썸
        /// </summary>
        public string Checksum
        {
            get
            {
                return string.Format("0x{0:x2}", this.checksum);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TCPHeader(sourceByteArray, countReceived)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sourceByteArray">소스 바이트 배열</param>
        /// <param name="countReceived">수신 카운트</param>
        public TCPHeader(byte[] sourceByteArray, int countReceived)
        {
            try
            {
                MemoryStream memoryStream = new MemoryStream(sourceByteArray, 0, countReceived);
                BinaryReader binaryReader = new BinaryReader(memoryStream);
                
                this.sourcePort            = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.destinationPort       = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.sequenceNumber        = (uint)IPAddress.NetworkToHostOrder(binaryReader.ReadInt32());
                this.acknowledgementNumber = (uint)IPAddress.NetworkToHostOrder(binaryReader.ReadInt32());
                this.dataOffsetAndFlags    = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.windowSize            = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.checksum              = (short)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.urgentPointer         = (ushort)IPAddress.NetworkToHostOrder(binaryReader.ReadInt16());
                this.headerLength          = (byte)(this.dataOffsetAndFlags >> 12);

                this.headerLength *= 4;
            }
            catch(Exception exception)
            {
                MessageBox.Show
                (
                    exception.Message,
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            }
        }

        #endregion
    }
}